@@ -31,10 +31,80 @@ DecorationBridge *findBridge(const QVariantList &args)
 }
 }
 
+BorderRadius::BorderRadius()
+{
+}
+
+BorderRadius::BorderRadius(qreal radius)
+    : BorderRadius(radius, radius, radius, radius)
+{
+}
+
+BorderRadius::BorderRadius(qreal topLeft, qreal topRight, qreal bottomRight, qreal bottomLeft)
+    : m_topLeft(topLeft)
+    , m_topRight(topRight)
+    , m_bottomRight(bottomRight)
+    , m_bottomLeft(bottomLeft)
+{
+}
+
+qreal BorderRadius::topLeft() const
+{
+    return m_topLeft;
+}
+
+qreal BorderRadius::topRight() const
+{
+    return m_topRight;
+}
+
+qreal BorderRadius::bottomRight() const
+{
+    return m_bottomRight;
+}
+
+qreal BorderRadius::bottomLeft() const
+{
+    return m_bottomLeft;
+}
+
+BorderOutline::BorderOutline()
+{
+}
+
+BorderOutline::BorderOutline(qreal thickness, const QColor &color, const BorderRadius &radius)
+    : m_thickness(thickness)
+    , m_color(color)
+    , m_radius(radius)
+{
+}
+
+bool BorderOutline::isNull() const
+{
+    return qFuzzyIsNull(m_thickness);
+}
+
+qreal BorderOutline::thickness() const
+{
+    return m_thickness;
+}
+
+QColor BorderOutline::color() const
+{
+    return m_color;
+}
+
+BorderRadius BorderOutline::radius() const
+{
+    return m_radius;
+}
+
 class DecorationStateData : public QSharedData
 {
 public:
     QMarginsF borders;
+    BorderRadius borderRadius;
+    BorderOutline borderOutline;
 };
 
 DecorationState::DecorationState()
@@ -66,6 +136,26 @@ void DecorationState::setBorders(const QMarginsF &borders)
     d->borders = borders;
 }
 
+BorderRadius DecorationState::borderRadius() const
+{
+    return d->borderRadius;
+}
+
+void DecorationState::setBorderRadius(const BorderRadius &radius)
+{
+    d->borderRadius = radius;
+}
+
+BorderOutline DecorationState::borderOutline() const
+{
+    return d->borderOutline;
+}
+
+void DecorationState::setBorderOutline(const BorderOutline &outline)
+{
+    d->borderOutline = outline;
+}
+
 Positioner::Positioner()
     : d(new PositionerData)
 {
@@ -303,6 +393,24 @@ void Decoration::setResizeOnlyBorders(const QMarginsF &borders)
     }
 }
 
+void Decoration::setBorderRadius(const BorderRadius &radius)
+{
+    if (d->next->borderRadius() != radius) {
+        setState([radius](DecorationState *state) {
+            state->setBorderRadius(radius);
+        });
+    }
+}
+
+void Decoration::setBorderOutline(const BorderOutline &outline)
+{
+    if (d->next->borderOutline() != outline) {
+        setState([outline](DecorationState *state) {
+            state->setBorderOutline(outline);
+        });
+    }
+}
+
 void Decoration::setTitleBar(const QRectF &rect)
 {
     if (d->titleBar != rect) {
@@ -402,6 +510,16 @@ qreal Decoration::resizeOnlyBorderBottom() const
     return d->resizeOnlyBorders.bottom();
 }
 
+BorderRadius Decoration::borderRadius() const
+{
+    return d->current->borderRadius();
+}
+
+BorderOutline Decoration::borderOutline() const
+{
+    return d->current->borderOutline();
+}
+
 QSizeF Decoration::size() const
 {
     const QMarginsF b = d->current->borders();
@@ -587,6 +705,12 @@ void Decoration::apply(std::shared_ptr<DecorationState> state)
     if (previous->borders() != state->borders()) {
         Q_EMIT bordersChanged();
     }
+    if (previous->borderRadius() != state->borderRadius()) {
+        Q_EMIT borderRadiusChanged();
+    }
+    if (previous->borderOutline() != state->borderOutline()) {
+        Q_EMIT borderOutlineChanged();
+    }
 
     Q_EMIT currentStateChanged(state);
 }
